/*
 * Copyright 2015, Yahoo Inc.
 * Copyrights licensed under the New BSD License.
 * See the accompanying LICENSE file for terms.
 */
import * as React from 'react';
import withIntl from './injectIntl';
const shallowEquals = require('shallow-equal/objects');
import { formatMessage as baseFormatMessage } from '../format';
import { invariantIntlContext, DEFAULT_INTL_CONFIG, createDefaultFormatters, } from '../utils';
const defaultFormatMessage = (descriptor, values) => {
    if (process.env.NODE_ENV !== 'production') {
        console.error('[React Intl] Could not find required `intl` object. <IntlProvider> needs to exist in the component ancestry. Using default message as fallback.');
    }
    return baseFormatMessage(Object.assign({}, DEFAULT_INTL_CONFIG, { locale: 'en' }), createDefaultFormatters(), descriptor, values);
};
export class BaseFormattedMessage extends React.Component {
    constructor(props) {
        super(props);
        if (!props.defaultMessage) {
            invariantIntlContext(props);
        }
    }
    shouldComponentUpdate(nextProps) {
        const { values } = this.props;
        const { values: nextValues } = nextProps;
        if (!shallowEquals(nextValues, values)) {
            return true;
        }
        // Since `values` has already been checked, we know they're not
        // different, so the current `values` are carried over so the shallow
        // equals comparison on the other props isn't affected by the `values`.
        let nextPropsToCheck = Object.assign({}, nextProps, { values });
        return !shallowEquals(this.props, nextPropsToCheck);
    }
    render() {
        const { formatMessage = defaultFormatMessage, textComponent: Text = React.Fragment, } = this.props.intl || {};
        const { id, description, defaultMessage, values, tagName: Component = Text, children, } = this.props;
        const descriptor = { id, description, defaultMessage };
        let nodes = formatMessage(descriptor, values);
        if (!Array.isArray(nodes)) {
            nodes = [nodes];
        }
        if (typeof children === 'function') {
            return children(...nodes);
        }
        if (Component) {
            // Needs to use `createElement()` instead of JSX, otherwise React will
            // warn about a missing `key` prop with rich-text message formatting.
            return React.createElement(Component, null, ...nodes);
        }
        return nodes;
    }
}
BaseFormattedMessage.defaultProps = {
    values: {},
};
export default withIntl(BaseFormattedMessage, { enforceContext: false });
